#include <linux/string.h>
#include <vsprintf.h>
#include <pcba_id.h>
#include <crc.h>
#include <common.h>
#include <i2c.h>
#include <aml_i2c.h>

#include "pcba_id_mini2.h"

// For version with LED-clock: 01Y00020...
static int parse_mini2_with_clock_pcba_id(const void *EEPROM, struct pcba_id_mini2 *p)
{
	unsigned crc;
	char extra_char;

	if (strlen(EEPROM) != 33 || sscanf(EEPROM, "01Y%5u%2s%2u%6u%6u%5u%4x%c",
			&p->model, p->manufacturer, &p->board_type, &p->revision, &p->date, &p->serial, &crc, &extra_char) != 7)
		return PARSE_RES__OTHER;

	if (cyg_crc16(EEPROM, 29) != crc)
		return PARSE_RES__WRONG_CRC;

	return PARSE_RES__SUCCESS;
}

int parse_eeprom_mini2(unsigned i2c_address, struct pcba_id_mini2 *pcba_id_mini2)
{
	unsigned char buffer[1 + 33 + 1] = {0};
	unsigned char* EEPROM = buffer + 1;

	struct i2c_msg msg[2] = {
		{i2c_address, 0, 1, buffer},
		{i2c_address, I2C_M_RD, 33, EEPROM}
	};

	if (i2c_set_bus_num(AML_I2C_MASTER_AO) || aml_i2c_xfer(msg, 2) != 2) {
		return PARSE_RES__OTHER;
	}

	return parse_mini2_with_clock_pcba_id(EEPROM, pcba_id_mini2);
}