/*
 * board/amlogic/yandexstation_mini2_a113x/yandexstation_mini2_a113x.c
 *
 * Copyright (C) 2015 Amlogic, Inc. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/
#include <common.h>
#include <malloc.h>
#include <errno.h>
#include <environment.h>
#include <fdt_support.h>
#include <libfdt.h>
#include <asm/arch/secure_apb.h>
#include <i2c.h>
#include <aml_i2c.h>
#ifdef CONFIG_AML_VPU
#include <vpu.h>
#endif
#include <vpp.h>
#ifdef CONFIG_AML_V2_FACTORY_BURN
#include <amlogic/aml_v2_burning.h>
#endif// #ifdef CONFIG_AML_V2_FACTORY_BURN
#ifdef CONFIG_AML_HDMITX20
#include <amlogic/hdmi.h>
#endif
#ifdef CONFIG_AML_LCD
#include <amlogic/aml_lcd.h>
#endif
#include <asm/arch/eth_setup.h>
#include <phy.h>
#include <asm/cpu_id.h>
#include <linux/mtd/partitions.h>
#include <linux/sizes.h>
#include <asm/arch/clock.h>
#include <asm/arch/board_id.h>
#include "pcba_id_mini2.h"
#include <pcba_id.h>

#ifdef CONFIG_GPIO_LONG_PRESS_NOTIFIER
#include <gpio_long_press.h>
#include <linux/list.h>
#endif

DECLARE_GLOBAL_DATA_PTR;

//new static eth setup
struct eth_board_socket*  eth_board_skt;

#ifdef CONFIG_YANDEXSTATION_MINI2_LED_ON_BOOT

#define LED_COLOR_NORMAL_BOOT 0
#define LED_COLOR_OTA 1
#define LED_COLOR_MUTE_LONG_PRESS 2
#define LED_COLOR_VOLTAGE_CMD_SUCCESS 3
#define LED_COLOR_VOLTAGE_CMD_ERROR 4

#define LED_COLOR_COUNT 5

static bool draw_led_pattern(int color);

#endif

#ifdef CONFIG_GPIO_LONG_PRESS_NOTIFIER
static bool g_mute_long_press_detected;
#endif

int serial_set_pin_port(unsigned long port_base)
{
    //UART in "Always On Module"
    //GPIOAO_0==tx,GPIOAO_1==rx
    //setbits_le32(P_AO_RTI_PIN_MUX_REG,3<<11);
    return 0;
}

int dram_init(void)
{
	gd->ram_size = PHYS_SDRAM_1_SIZE;
	return 0;
}

/* secondary_boot_func
 * this function should be write with asm, here, is is only for compiling pass
 * */
void secondary_boot_func(void)
{
}
void internalPhyConfig(struct phy_device *phydev)
{
	/*Enable Analog and DSP register Bank access by*/
	phy_write(phydev, MDIO_DEVAD_NONE, 0x14, 0x0000);
	phy_write(phydev, MDIO_DEVAD_NONE, 0x14, 0x0400);
	phy_write(phydev, MDIO_DEVAD_NONE, 0x14, 0x0000);
	phy_write(phydev, MDIO_DEVAD_NONE, 0x14, 0x0400);
	/*Write Analog register 23*/
	phy_write(phydev, MDIO_DEVAD_NONE, 0x17, 0x8E0D);
	phy_write(phydev, MDIO_DEVAD_NONE, 0x14, 0x4417);
	/*Enable fractional PLL*/
	phy_write(phydev, MDIO_DEVAD_NONE, 0x17, 0x0005);
	phy_write(phydev, MDIO_DEVAD_NONE, 0x14, 0x5C1B);
	//Programme fraction FR_PLL_DIV1
	phy_write(phydev, MDIO_DEVAD_NONE, 0x17, 0x029A);
	phy_write(phydev, MDIO_DEVAD_NONE, 0x14, 0x5C1D);
	//## programme fraction FR_PLL_DiV1
	phy_write(phydev, MDIO_DEVAD_NONE, 0x17, 0xAAAA);
	phy_write(phydev, MDIO_DEVAD_NONE, 0x14, 0x5C1C);
}

#ifdef CONFIG_CMD_NET
static void setup_net_chip(void)
{
	eth_aml_reg0_t eth_reg0;
	writel(0x11111111, P_PERIPHS_PIN_MUX_8);
	writel(0x111111, P_PERIPHS_PIN_MUX_9);
	eth_reg0.d32 = 0;
	eth_reg0.b.phy_intf_sel = 1;
	eth_reg0.b.rx_clk_rmii_invert = 0;
	eth_reg0.b.rgmii_tx_clk_src = 0;
	eth_reg0.b.rgmii_tx_clk_phase = 1;
	eth_reg0.b.rgmii_tx_clk_ratio = 4;
	eth_reg0.b.phy_ref_clk_enable = 1;
	eth_reg0.b.clk_rmii_i_invert = 0;
	eth_reg0.b.clk_en = 1;
	eth_reg0.b.adj_enable = 0;
	eth_reg0.b.adj_setup = 0;
	eth_reg0.b.adj_delay = 0;
	eth_reg0.b.adj_skew = 0;
	eth_reg0.b.cali_start = 0;
	eth_reg0.b.cali_rise = 0;
	eth_reg0.b.cali_sel = 0;
	eth_reg0.b.rgmii_rx_reuse = 0;
	eth_reg0.b.eth_urgent = 0;
	setbits_le32(P_PREG_ETH_REG0, eth_reg0.d32);// rmii mode
	setbits_le32(HHI_GCLK_MPEG1,1<<3);
	/* power on memory */
	clrbits_le32(HHI_MEM_PD_REG0, (1 << 3) | (1<<2));

	/* hardware reset ethernet phy : gpioY15 connect phyreset pin*/
	udelay(1000);
	setbits_le32(P_PREG_PAD_GPIO1_EN_N, 1 << 15);

}

extern struct eth_board_socket* eth_board_setup(char *name);
extern int designware_initialize(ulong base_addr, u32 interface);

int board_eth_init(bd_t *bis)
{
	setup_net_chip();
	udelay(1000);
	designware_initialize(ETH_BASE, PHY_INTERFACE_MODE_RMII);
	return 0;
}
#endif

#if CONFIG_AML_SD_EMMC
#include <mmc.h>
#include <asm/arch/sd_emmc.h>
static int  sd_emmc_init(unsigned port)
{
    switch (port)
	{
		case SDIO_PORT_A:
			break;
		case SDIO_PORT_B:
			//todo add card detect
			//setbits_le32(P_PREG_PAD_GPIO5_EN_N,1<<29);//CARD_6
			break;
		case SDIO_PORT_C:
			//enable pull up
			//clrbits_le32(P_PAD_PULL_UP_REG3, 0xff<<0);
			break;
		default:
			break;
	}

	return cpu_sd_emmc_init(port);
}

extern unsigned sd_debug_board_1bit_flag;
static int  sd_emmc_detect(unsigned port)
{
	int ret;
    switch (port) {

	case SDIO_PORT_A:
		break;
	case SDIO_PORT_B:
			/*axg skt using GPIOX_6 as CD, no debug board anymore*/
			clrbits_le32(P_PERIPHS_PIN_MUX_4, 0xF000000);
			setbits_le32(P_PREG_PAD_GPIO2_EN_N, 1 << 6);
			ret = readl(P_PREG_PAD_GPIO2_I) & (1 << 6) ? 0 : 1;
			printf("%s\n", ret ? "card in" : "card out");
		#if 0 /* no default card on board. */
			if ((readl(P_PERIPHS_PIN_MUX_6) & (3 << 8))) { //if uart pinmux set, debug board in
				if (!(readl(P_PREG_PAD_GPIO2_I) & (1 << 24))) {
					printf("sdio debug board detected, sd card with 1bit mode\n");
					sd_debug_board_1bit_flag = 1;
				}
				else{
					printf("sdio debug board detected, no sd card in\n");
					sd_debug_board_1bit_flag = 0;
					return 1;
				}
			}
		#endif
		break;
	default:
		break;
	}
	return 0;
}

static void sd_emmc_pwr_prepare(unsigned port)
{
	cpu_sd_emmc_pwr_prepare(port);
}

static void sd_emmc_pwr_on(unsigned port)
{
    switch (port)
	{
		case SDIO_PORT_A:
			break;
		case SDIO_PORT_B:
//            clrbits_le32(P_PREG_PAD_GPIO5_O,(1<<31)); //CARD_8
//            clrbits_le32(P_PREG_PAD_GPIO5_EN_N,(1<<31));
			/// @todo NOT FINISH
			break;
		case SDIO_PORT_C:
			break;
		default:
			break;
	}
	return;
}
static void sd_emmc_pwr_off(unsigned port)
{
	/// @todo NOT FINISH
    switch (port)
	{
		case SDIO_PORT_A:
			break;
		case SDIO_PORT_B:
//            setbits_le32(P_PREG_PAD_GPIO5_O,(1<<31)); //CARD_8
//            clrbits_le32(P_PREG_PAD_GPIO5_EN_N,(1<<31));
			break;
		case SDIO_PORT_C:
			break;
				default:
			break;
	}
	return;
}

// #define CONFIG_TSD      1
static void board_mmc_register(unsigned port)
{
	struct aml_card_sd_info *aml_priv=cpu_sd_emmc_get(port);
    if (aml_priv == NULL)
		return;

	aml_priv->sd_emmc_init=sd_emmc_init;
	aml_priv->sd_emmc_detect=sd_emmc_detect;
	aml_priv->sd_emmc_pwr_off=sd_emmc_pwr_off;
	aml_priv->sd_emmc_pwr_on=sd_emmc_pwr_on;
	aml_priv->sd_emmc_pwr_prepare=sd_emmc_pwr_prepare;
	aml_priv->desc_buf = malloc(NEWSD_MAX_DESC_MUN*(sizeof(struct sd_emmc_desc_info)));

	if (NULL == aml_priv->desc_buf)
		printf(" desc_buf Dma alloc Fail!\n");
	else
		printf("aml_priv->desc_buf = 0x%p\n",aml_priv->desc_buf);

	sd_emmc_register(aml_priv);
}
int board_mmc_init(bd_t	*bis)
{
#ifdef CONFIG_VLSI_EMULATOR
	//board_mmc_register(SDIO_PORT_A);
#else
	//board_mmc_register(SDIO_PORT_B);
#endif
	board_mmc_register(SDIO_PORT_B);
	board_mmc_register(SDIO_PORT_C);
//	board_mmc_register(SDIO_PORT_B1);
	return 0;
}
#endif

struct aml_i2c_platform g_aml_i2c_plat = {
};
/* multi i2c bus */
struct aml_i2c_platform g_aml_i2c_ports[] = {
	/* MASTER AO */
	{
		.wait_count         = 1000000,
		.wait_ack_interval  = 5,
		.wait_read_interval = 5,
		.wait_xfer_interval = 5,
		.master_no          = AML_I2C_MASTER_AO,
		.use_pio            = 0,
		.master_i2c_speed   = CONFIG_SYS_I2C_SPEED,
		.master_ao_pinmux = {
			.scl_reg    = (unsigned long)MESON_I2C_MASTER_AO_GPIOAO_10_REG,
			.scl_bit    = MESON_I2C_MASTER_AO_GPIOAO_10_BIT,
			.sda_reg    = (unsigned long)MESON_I2C_MASTER_AO_GPIOAO_11_REG,
			.sda_bit    = MESON_I2C_MASTER_AO_GPIOAO_11_BIT,
		}
	},

	/* MASTER B */
	{
		.wait_count         = 1000000,
		.wait_ack_interval  = 5,
		.wait_read_interval = 5,
		.wait_xfer_interval = 5,
		.master_no          = AML_I2C_MASTER_B,
		.use_pio            = 0,
		.master_i2c_speed   = CONFIG_SYS_I2C_SPEED,
		.master_b_pinmux = {
			.scl_reg    = (unsigned long)MESON_I2C_MASTER_B_GPIOZ_8_REG,
			.scl_bit    = MESON_I2C_MASTER_B_GPIOZ_8_BIT,
			.sda_reg    = (unsigned long)MESON_I2C_MASTER_B_GPIOZ_9_REG,
			.sda_bit    = MESON_I2C_MASTER_B_GPIOZ_9_BIT,
		}
	},

	/* End of list */
	{
		.master_i2c_speed = 0
	}
};


static void mini2_i2c_init(void)
{
	//Amlogic I2C controller initialized
	//note: it must be call before any I2C operation
	//aml_i2c_init();
	extern void aml_i2c_set_ports(struct aml_i2c_platform *i2c_plat);
	aml_i2c_set_ports(g_aml_i2c_ports);

	udelay(10);
}

#if defined(CONFIG_BOARD_EARLY_INIT_F)
int board_early_init_f(void){
	/*add board early init function here*/
	return 0;
}
#endif

#ifdef CONFIG_USB_XHCI_AMLOGIC_GXL
#include <asm/arch/usb-new.h>
#include <asm/arch/gpio.h>
#define CONFIG_GXL_USB_U2_PORT_NUM	4
#define CONFIG_GXL_USB_U3_PORT_NUM	0

static void gpio_set_vbus_power(char is_power_on)
{
	if (is_power_on) {
		clrbits_le32(P_AO_GPIO_O_EN_N, (1<<5));
		setbits_le32(P_AO_GPIO_O_EN_N, (1<<21));
	} else {
		clrbits_le32(P_AO_GPIO_O_EN_N, (1<<5));
		clrbits_le32(P_AO_GPIO_O_EN_N, (1<<21));
	}
}

struct amlogic_usb_config g_usb_config_GXL_skt={
	CONFIG_GXL_XHCI_BASE,
	USB_ID_MODE_HARDWARE,
	gpio_set_vbus_power,
	CONFIG_GXL_USB_PHY2_BASE,
	CONFIG_GXL_USB_PHY3_BASE,
	CONFIG_GXL_USB_U2_PORT_NUM,
	CONFIG_GXL_USB_U3_PORT_NUM,
};
#endif /*CONFIG_USB_XHCI_AMLOGIC*/

#ifdef CONFIG_AML_HDMITX20
static void hdmi_tx_set_hdmi_5v(void)
{
}
#endif

/*
 * mtd nand partition table, only care the size!
 * offset will be calculated by nand driver.
 */
#ifdef CONFIG_AML_MTD
static struct mtd_partition normal_partition_info[] = {
#ifdef CONFIG_DISCRETE_BOOTLOADER
    /* MUST NOT CHANGE this part unless u know what you are doing!
     * inherent parition for descrete bootloader to store fip
     * size is determind by TPL_SIZE_PER_COPY*TPL_COPY_NUM
     * name must be same with TPL_PART_NAME
     */
    {
        .name = "tpl",
        .offset = 0,
        .size = 0,
    },
#endif
#ifdef CONFIG_AB_SYSTEM
    {
        .name = "boot_a",
        .offset = 0,
        .size = 15*SZ_1M,
    },
    {
        .name = "boot_b",
        .offset = 0,
        .size = 15*SZ_1M,
    },
    {
        .name = "system_a",
        .offset = 0,
        .size = 150*SZ_1M,
    },
    {
        .name = "system_b",
        .offset = 0,
        .size = 150*SZ_1M,
    },
 #else
    {
        .name = "boot",
        .offset = 0,
        .size = 10*SZ_1M,
    },
    {
        .name = "system",
        .offset = 0,
        .size = 80*SZ_1M,
    },
    {
        .name = "nv",
        .offset = 0,
        .size = 1*SZ_1M,
    },
#endif
    {
        .name = "bkupgrade",
        .offset = 0,
        .size = 105*SZ_1M,
    },
    {
        .name = "recovery",
        .offset = 0,
        .size = 50*SZ_1M,
    },
};

struct mtd_partition *get_aml_mtd_partition(uint64_t nand_size)
{
	return normal_partition_info;
}

int get_aml_partition_count(uint64_t nand_size)
{
	return ARRAY_SIZE(normal_partition_info);
}
#endif /* CONFIG_AML_MTD */
void power_save_pre(void)
{
	/*Close MIPI clock*/
	clrbits_le32(P_HHI_MIPI_CNTL0, 0xffffffff);
	clrbits_le32(P_HHI_MIPI_CNTL1, 0xffffffff);
	clrbits_le32(P_HHI_MIPI_CNTL2, 0xffdfffff);
}

#if 0
static int led_extern_change_i2c_bus(unsigned aml_i2c_bus)
{
	int ret = 0;
	extern struct aml_i2c_platform g_aml_i2c_plat;

	g_aml_i2c_plat.master_no = aml_i2c_bus;
	ret = aml_i2c_init();

	return ret;
}
#endif

static int do_yandex_io_run_recovery(void)
{
	printf("Yandex.IO boot to recovery\n");

	run_command("run recovery_from_flash", 0);
	printf("run_command for cmd:run recovery_from_flash failed.\n");
	return -1;
}

static bool usb_on_hp(bool on)
{
	static bool old_value = false;
	const bool ret = old_value;

	if (on != old_value) {
		if (on)
			run_command("gpio set GPIOA_20", 0);
		else
			run_command("gpio clear GPIOA_20", 0);
		old_value = on;
	}

	return ret;
}

static int do_yandex_io_check_boot_mode(cmd_tbl_t *cmdtp, int flag, int argc, char *const argv[])
{
	const char *factory_mode = getenv("factory_mode");

	if (g_mute_long_press_detected) {
		bool old_value;
		printf("Trying auto burn (long mute button press detected)\n");
		old_value = usb_on_hp(true);
		run_command("update 700 750", 0);
		usb_on_hp(old_value); // restore
	}

	if (factory_mode) {
		if (!strcmp(factory_mode, "1")) {
			if (g_mute_long_press_detected) {
				run_command("setenv bootargs ${bootargs} do_hw_test_on_startup", 0);
				return 0;
			} else
				run_command("fastboot", 0);
		} else if (!strcmp(factory_mode, "3"))
			run_command("update", 0);
	}

	printf("Yandex.IO check recovery\n");
	if (g_mute_long_press_detected) {
		printf("Execute recovery boot requested by a long mute button press\n");
		run_command("ubi part bkupgrade", 0);
		run_command("ubi remove bkupgrade", 0);
		run_command("ubi create bkupgrade", 0);
		do_yandex_io_run_recovery();
	} else if (getenv_yesno("boot_to_recovery") == 1) {
		printf("Execute recovery boot requested by an environment variable\n");
#ifdef CONFIG_YANDEXSTATION_MINI2_LED_ON_BOOT
		draw_led_pattern(LED_COLOR_OTA);
#endif
		do_yandex_io_run_recovery();
	} else {
		printf("Don't have any recovery requests, skip...\n");
	}

	return 0;
}

U_BOOT_CMD(yandex_io_check_boot_mode, 1, 0, do_yandex_io_check_boot_mode,
	"Yandex.IO check factory/recovery request and boot into it if needed",
	"yandex_io_check_boot_mode\n")

#ifdef CONFIG_YANDEXSTATION_MINI2_LED_ON_BOOT

#define DEVICE_COLOR_BLACK 0
#define DEVICE_COLOR_BLUE  1
#define DEVICE_COLOR_RED   2

#define DEVICE_COLOR_COUNT 3

static const char* get_device_id(void) {
    const char* device_id = getenv("deviceid");
    if (device_id) {
        return device_id;
    }

    unsigned char buffer[64];
    char read_did_cmd[128];

    snprintf(
        read_did_cmd, sizeof(read_did_cmd),
        "if keyman init 0x1234; then "\
            "keyman read deviceid 0x%p str;"\
        "fi;",
        buffer
    );

    run_command(read_did_cmd, 0);

    return getenv("deviceid");
}

static int get_device_color(void) {
    const char* device_id = get_device_id();
    char device_color = 'K';

    printf("device_id %s\n", device_id);

    if (device_id) {
        switch (strlen(device_id)) {
            case 32: // Old device id
                device_color = device_id[1];
                break;
            case 14: // New device id
                device_color = device_id[13];
                break;
            default:
                break;
        }
    }

    switch (device_color) {
        case 'K':
        default:
            return DEVICE_COLOR_BLACK;
        case 'B':
            return DEVICE_COLOR_BLUE;
        case 'R':
            return DEVICE_COLOR_RED;
    }
}

static const uint32_t led_colors[LED_COLOR_COUNT][DEVICE_COLOR_COUNT] = {
    {0x6ef40e, 0xff620f, 0x11f5b3}, // LED_COLOR_NORMAL_BOOT
    {0xdddddd, 0xff2950, 0x042bff}, // LED_COLOR_OTA
    {0xff00e6, 0xff014f, 0x0600ff}, // LED_COLOR_MUTE_LONG_PRESS
    {0x0000ff, 0x0000ff, 0x0000ff}, // LED_COLOR_VOLTAGE_CMD_SUCCESS
    {0xff0000, 0xff0000, 0xff0000}  // LED_COLOR_VOLTAGE_CMD_ERROR
};

static uint32_t get_led_color(int type) {
    const int device_color = get_device_color();

    printf("device_color %d\n", device_color);

    return led_colors[type][device_color];
}

#define LED_COUNT 6

static const unsigned char led_mapping[3 * LED_COUNT] = {
	1, 2, 3, 4, 5, 6, 7, 8, 9,
	11, 10, 12, 13, 14, 15, 18, 17, 16
};

static bool draw_led_color(uint32_t color)
{
	const unsigned char LED_RING_I2C_CHIP = 0x3C;
	int ret, i, led;
	unsigned char buffer[1 + 3 * LED_COUNT];
	struct i2c_msg led_msg = {LED_RING_I2C_CHIP, 0, 0, buffer};

	ret = i2c_set_bus_num(AML_I2C_MASTER_AO);
	if (ret)
		goto error;

	buffer[0] = 0x4F; // Reset Register
	buffer[1] = 0; // reset all registers to default value
	led_msg.len = 2;
	ret = aml_i2c_xfer(&led_msg, 1);
	if (ret != 1)
		goto error;

	buffer[0] = 0x4B; // Output Frequency Setting Register
	buffer[1] = 1; // 22kHz
	led_msg.len = 2;
	ret = aml_i2c_xfer(&led_msg, 1);
	if (ret != 1)
		goto error;

	buffer[0] = 0; // Shutdown Register
	buffer[1] = 1; // Normal operation
	led_msg.len = 2;
	ret = aml_i2c_xfer(&led_msg, 1);
	if (ret != 1)
		goto error;

	buffer[0] = 3; // PWM Registers for LED1 and on
	for (led = 0; led < LED_COUNT; led++) {
		uint32_t gbr = color;

		for (i = 0; i < 3; i++) {
			buffer[led_mapping[led * 3 + i]] = gbr % 256;
			gbr >>= 8;
		}
	}

	led_msg.len = sizeof buffer;
	ret = aml_i2c_xfer(&led_msg, 1);
	if (ret != 1)
		goto error;

	buffer[0] = 0x28; // LED Control Registers for LED1 and on
	memset(buffer + 1, 1, sizeof buffer - 1); // set all LEDs on with max. current
	led_msg.len = sizeof buffer;
	ret = aml_i2c_xfer(&led_msg, 1);
	if (ret != 1)
		goto error;

	buffer[0] = 0x25; // PWM Update Register
	buffer[1] = 0; // update values for PWM and the LED Control
	led_msg.len = 2;
	ret = aml_i2c_xfer(&led_msg, 1);
	if (ret != 1)
		goto error;

	return true;

error:
	printf("Error setting color 0x%x for all the LEDs\n", color);
	return false;
}

static bool draw_led_pattern(int type)
{
	const uint32_t color = get_led_color(type);
	printf("draw_led_pattern type %d, color 0x%x\n", type, color);

	return draw_led_color(color);
}

static int do_led_ring_fill(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
	if (argc < 2) {
		return cmd_usage(cmdtp);
	}

	uint32_t rgb_color = simple_strtoul(argv[1], NULL, 16);

	// draw_led_color() expects rrBBgg, not rrGGbb, so convert color
	uint32_t rbg_color = (rgb_color & 0xff0000) | ((rgb_color & 0xff00) >> 8) |
				((rgb_color & 0xff) << 8);

	if (draw_led_color(rbg_color) != true) {
		return CMD_RET_FAILURE;
	}

	return CMD_RET_SUCCESS;
}

static cmd_tbl_t cmd_led_ring_sub[] = {
	U_BOOT_CMD_MKENT(fill, 2, 1, do_led_ring_fill,
		"fill LED ring", "<COLOR> - fill LED ring by single color (COLOR is hex number: RRGGBB)"),
};

static int do_led_ring(cmd_tbl_t *cmdtp, int flag, int argc, char *const argv[])
{
	if (argc < 2) {
		return CMD_RET_USAGE;
	}

	cmd_tbl_t *c;
	// strip off leading "led_ring" command argument
	argc--;
	argv++;

	c = find_cmd_tbl(argv[0], &cmd_led_ring_sub[0], ARRAY_SIZE(cmd_led_ring_sub));
	if (c == NULL) {
		return CMD_RET_USAGE;
	}

	return c->cmd(c, flag, argc, argv);
}

U_BOOT_CMD(led_ring, 3, 1, do_led_ring,
	"Tool for controling led ring",
	"fill COLOR - fill led ring by single color (COLOR is hex number: RRGGBB)\n")

#endif

#ifdef CONFIG_GPIO_LONG_PRESS_NOTIFIER
static LIST_HEAD(notifiers_list);

static int long_press_cb(void *data)
{
	printf("mute button long press detected\n");
	g_mute_long_press_detected = true;

	return 0;
}

static struct gpio_long_press_notifier on_mute_pressed_notifier_arr[] = {
	{
		.function = long_press_cb,
	},
};
#endif /* CONFIG_GPIO_LONG_PRESS_NOTIFIER */

int board_init(void)
{
	/* Enable LED controller */
	run_command("gpio set GPIOAO_3", 1);

	mini2_i2c_init();

#ifdef CONFIG_AML_V2_FACTORY_BURN
	aml_try_factory_usb_burning(0, gd->bd);
#endif// #ifdef CONFIG_AML_V2_FACTORY_BURN

#ifdef CONFIG_USB_XHCI_AMLOGIC_GXL
	board_usb_init(&g_usb_config_GXL_skt,BOARD_USB_MODE_HOST);
#endif /*CONFIG_USB_XHCI_AMLOGIC*/

#ifdef CONFIG_AML_NAND
	extern int amlnf_init(unsigned char flag);
	amlnf_init(0);
#endif

#ifdef CONFIG_GPIO_LONG_PRESS_NOTIFIER
	{
		int ret;
		int i;

		for (i = 0; i < ARRAY_SIZE(on_mute_pressed_notifier_arr); i++) {
			list_add_tail(&on_mute_pressed_notifier_arr[i].list,
				&notifiers_list);
		}

		ret = gpio_long_press_check(22 /* GPIOZ_8 */, 5000 /* ms */,
			&notifiers_list);
		if (ret < 0) {
			printf("Error checking long press button\n");
			return ret;
		}
	}
#endif

	if (get_cpu_id().package_id == MESON_CPU_PACKAGE_ID_A113X)
		power_save_pre();
	return 0;
}

#ifdef CONFIG_BOARD_LATE_INIT

int board_late_init(void){
	char boardid[64] = {0};
	char tempBuf[64] = {0};
	unsigned board_id = get_board_id();
	unsigned pcb_id = board_id;
	const char *factory_mode = getenv("factory_mode");

	usb_on_hp(factory_mode && strcmp(factory_mode, "0"));

#ifdef CONFIG_SILENT_CONSOLE
	run_command("if printenv silent; then "
				"if itest ${silent} == 1; then silent on; fi; "
			"else "
				"setenv silent 1; silent on; saveenv; "
			"fi;",
				0);
#endif

#if (defined (A98_EATON) || defined(A98_ORION)) || defined(A98_ORION_MIDEA) || defined(YANDEXMINI_HW_PLATFORM)
	printf("project EATON or ORION force to A98L\n");
	board_id = 12;
#elif defined (A98_FRIENDSMINI_ALEXA)
    board_id = 8;
#else
    //not onboard project, set pcb_id to 0
    pcb_id = 0;
#endif	
	printf("board id is : %d\n",board_id);
	printf("PCB id is : %d\n",pcb_id);
	sprintf(boardid, "%d" , board_id);
	setenv("Board_id", boardid);
	sprintf(tempBuf, "%d" , pcb_id);
	setenv("PCB_id", tempBuf);
	saveenv();

	//update env before anyone using it
	run_command("get_rebootmode; echo reboot_mode=${reboot_mode}; "\
			"if test ${reboot_mode} = factory_reset; then "\
			"defenv_reserv factory_mode aml_dt;setenv upgrade_step 2;save; fi;", 0);
	run_command("if itest ${upgrade_step} == 1; then "\
				"defenv_reserv factory_mode; setenv upgrade_step 2; saveenv; fi;", 0);
	/*add board late init function here*/

        // Get ChipID and put to env variable
#ifdef  CONFIG_AML_SERIAL
	uint8_t buff[16] = {0};
	char serial[128] = {0};

	if (get_chip_id(&buff[0], sizeof(buff)) == 0) {
		snprintf(serial, sizeof(serial), "%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x",
			buff[0], buff[1], buff[2], buff[3], buff[4], buff[5], buff[6], buff[7],
			buff[8], buff[9], buff[10], buff[11], buff[12], buff[13], buff[14], buff[15]);
		printf("device serial: %s\n", serial);
		if (setenv("aml_serial", serial)) {
			printf("error: set variable [aml_serial] fail\n");
		}
	}
#endif

	int ret;
	ret = run_command("store dtb read $dtb_mem_addr", 1);
	if (ret) {
		printf("%s(): [store dtb read $dtb_mem_addr] fail\n", __func__);
		#ifdef CONFIG_DTB_MEM_ADDR
		char cmd[64];
		printf("load dtb to %x\n", CONFIG_DTB_MEM_ADDR);
		sprintf(cmd, "store dtb read %x", CONFIG_DTB_MEM_ADDR);
		ret = run_command(cmd, 1);
		if (ret) {
			printf("%s(): %s fail\n", __func__, cmd);
		}
		#endif
	}

	/* load unifykey */
	//run_command("keyunify init 0x1234", 0);

#ifdef CONFIG_YANDEXSTATION_MINI2_LED_ON_BOOT
    if (g_mute_long_press_detected) {
		draw_led_pattern(LED_COLOR_MUTE_LONG_PRESS);
    } else {
        draw_led_pattern(LED_COLOR_NORMAL_BOOT);
    }
#endif

#ifdef CONFIG_AML_VPU
	vpu_probe();
#endif
	//vpp_init();
#ifdef CONFIG_AML_HDMITX20
	hdmi_tx_set_hdmi_5v();
	hdmi_tx_init();
#endif
#ifdef CONFIG_AML_LCD
	lcd_probe();
#endif

#ifdef CONFIG_AML_V2_FACTORY_BURN
	/*aml_try_factory_sdcard_burning(0, gd->bd);*/
#endif// #ifdef CONFIG_AML_V2_FACTORY_BURN

	return 0;
}
#endif

#ifdef CONFIG_AML_TINY_USBTOOL
int usb_get_update_result(void)
{
	unsigned long upgrade_step;
	upgrade_step = simple_strtoul (getenv ("upgrade_step"), NULL, 16);
	printf("upgrade_step = %d\n", (int)upgrade_step);
	if (upgrade_step == 1)
	{
		run_command("defenv", 1);
		run_command("setenv upgrade_step 2", 1);
		run_command("saveenv", 1);
		return 0;
	}
	else
	{
		return -1;
	}
}
#endif

phys_size_t get_effective_memsize(void)
{
	// >>16 -> MB, <<20 -> real size, so >>16<<20 = <<4
#if defined(CONFIG_SYS_MEM_TOP_HIDE)
	return (((readl(AO_SEC_GP_CFG0)) & 0xFFFF0000) << 4) - CONFIG_SYS_MEM_TOP_HIDE;
#else
	return (((readl(AO_SEC_GP_CFG0)) & 0xFFFF0000) << 4);
#endif
}

#ifdef CONFIG_MULTI_DTB
int checkhw(char * name)
{
	strcpy(name, "axg_mini2_nau");

	unsigned i2c_addr = 0x50;

	unsigned pcba_id_version = 0;
	unsigned board_type;

	// First we try regular pcba id lib
	struct pcba_id pcba_id;
	if (parse_eeprom(i2c_addr, &pcba_id) == PARSE_RES__SUCCESS) {
		pcba_id_version = pcba_id.struct_version;
		board_type = pcba_id.board_code;
	} else {
		// Then try old parser (with 33byte pcba v1 support)
		struct pcba_id_mini2 pcba_id_mini2;
		if(parse_eeprom_mini2(i2c_addr, &pcba_id_mini2) == PARSE_RES__SUCCESS) {
			pcba_id_version = 1;
			board_type = pcba_id_mini2.board_type;
		}
	}

	if (pcba_id_version == 1) {
		if (board_type == BOARD_TYPE_EVEREST)
			strcpy(name, "axg_mini2_es");
		if (board_type == BOARD_TYPE_UNISOC)
			strcpy(name, "axg_mini2_uni");
		if (board_type == BOARD_TYPE_REVF_NXP)
			strcpy(name, "axg_mini2_revf-nxp");
		if (board_type == BOARD_TYPE_REVF_UNISOC)
			strcpy(name, "axg_mini2_revf-uni");
			
	} else if (pcba_id_version == 2) {
		// In pcba id v2, assembly field is used to determine dts
		// For now, devices with v2 can only be axg_mini2_revf-uni
		strcpy(name, "axg_mini2_revf-uni");
	}

	setenv("aml_dt", name);
	return 0;
}
#endif

const char * const _env_args_reserve_[] =
{
		"aml_dt",
		"firstboot",

		NULL//Keep NULL be last to tell END
};
