#include "led_ctrl.h"

#include <errno.h>

static struct led_ctrl g_led_ctrl;

int led_ctrl_register(unsigned int led_count, const struct led_ctrl_ops *ops) {
    int ret = 0;

    LED_CTRL_DBG("Registering LED ctrl\n");

    if (g_led_ctrl.ops) {
        LED_CTRL_ERR("LED control instance already initalized\n");
        return -EPERM;
    }

    if (!ops) {
        LED_CTRL_ERR("LED ops is NULL\n");
        return -EINVAL;
    }

    g_led_ctrl.led_count = led_count;
    g_led_ctrl.ops = ops;

    if (g_led_ctrl.ops->led_init) {
        ret = g_led_ctrl.ops->led_init(&g_led_ctrl);
    }

    LED_CTRL_DBG("Registering LED ctrl finished successfully\n");

    return ret;
};

int led_ctrl_unregister(void) {
    int ret = 0;

    LED_CTRL_DBG("Unregistering LED ctrl\n");

    if (g_led_ctrl.ops->led_deinit) {
        ret = g_led_ctrl.ops->led_deinit(&g_led_ctrl);
    }

    LED_CTRL_DBG("Unregistering LED ctrl finished with ret = %d\n", ret);

    return ret;
};

int led_ctrl_enable_led(led_t led, color_t color) {
    int ret = 0;

    LED_CTRL_DBG("Enable LED 0x%x with color 0x%x\n",
        (unsigned int)led,
        (unsigned int)color
    );

    if (led >= g_led_ctrl.led_count) {
        LED_CTRL_ERR("LED 0x%x is above overall LED count 0x%x\n",
            (unsigned int)led, (unsigned int)g_led_ctrl.led_count
        );

        return -1;
    }

    if (g_led_ctrl.ops->led_enable) {
        ret = g_led_ctrl.ops->led_enable(&g_led_ctrl, led, color);
    }

    LED_CTRL_DBG("Enable LED 0x%x finished with ret = %d\n",
        (unsigned int)led, ret
    );

    return ret;
};

int led_ctrl_disable_led(led_t led) {
    int ret = 0;

    LED_CTRL_DBG("Disable LED 0x%x\n", (unsigned int)led);

    if (led >= g_led_ctrl.led_count) {
        LED_CTRL_ERR("LED 0x%x is above overall LED count 0x%x\n",
            led, g_led_ctrl.led_count
        );

        return -1;
    }

    if (g_led_ctrl.ops->led_disable) {
        ret = g_led_ctrl.ops->led_disable(&g_led_ctrl, led);
    }

    LED_CTRL_DBG("Disable LED 0x%x finished with ret = %d\n",
        (unsigned int)led, ret
    );

    return ret;
};

int led_ctrl_enable_all(color_t color) {
    int ret = 0;
    int i;

    LED_CTRL_DBG("Enable all LEDs\n");

    for (i = 0; i < g_led_ctrl.led_count; i++) {
        ret |= led_ctrl_enable_led((led_t)i, color);
    }

    LED_CTRL_DBG("Enable all LEDs finished\n");

    return ret;
}

int led_ctrl_disable_all(void) {
    int ret = 0;
    int i;

    LED_CTRL_DBG("Disable all LEDs\n");

    for (i = 0; i < g_led_ctrl.led_count; i++) {
        ret |= led_ctrl_disable_led((led_t)i);
    }

    LED_CTRL_DBG("Disable all LEDs finished\n");

    return ret;
}

int led_ctrl_transact(void) {
    int ret = 0;

    LED_CTRL_DBG("Performing LEDs transact\n");

    if (g_led_ctrl.ops->led_transact) {
        ret = g_led_ctrl.ops->led_transact(&g_led_ctrl);
    }

    LED_CTRL_DBG("Performing LEDs transact finished with ret = %d\n", ret);

    return ret;
}
