#ifndef __LED_CTRL_H__
#define __LED_CTRL_H__

#include <common.h>

// #define DEBUG

#ifdef DEBUG
#define LED_CTRL_DBG(fmt, ...) \
    printf("[led_ctrl %s]: "fmt, __FUNCTION__, ##__VA_ARGS__)
#else
#define LED_CTRL_DBG(fmt, ...)
#endif

#define LED_CTRL_ERR(fmt, ...) \
    printf("[led_ctrl: %s ERROR]: "fmt, __FUNCTION__, ##__VA_ARGS__)

typedef unsigned int led_t;
typedef u32 color_t;

struct led_ctrl;

struct led_ctrl_ops {
    int (*led_init)(const struct led_ctrl *);
    int (*led_deinit)(const struct led_ctrl *);

    int (*led_enable)(const struct led_ctrl *, led_t, color_t);
    int (*led_disable)(const struct led_ctrl *, led_t);

    int (*led_transact)(const struct led_ctrl *);
};

struct led_ctrl {
    const struct led_ctrl_ops *ops;
    led_t led_count;
    void *data;
};


extern int led_ctrl_register(unsigned int led_count, const struct led_ctrl_ops *ops);
extern int led_ctrl_unregister(void);

extern int led_ctrl_enable_led(led_t led, color_t color);
extern int led_ctrl_disable_led(led_t led);

extern int led_ctrl_enable_all(color_t color);
extern int led_ctrl_disable_all(void);

extern int led_ctrl_transact(void);

#endif /* __LED_CTRL_H__ */
