#ifndef _PCBA_ID_H_
#define _PCBA_ID_H_

// Size of pcba_id fields substring
#define FIELD_SIZE__STRUCT_VERSION			2
#define MAX_FIELD_SIZE						64
#define FIELD_SIZE__DATE					6
#define FIELD_SIZE__CHECKSUM				4

#define STRUCT_VERSION__DEFAULT				1
#define PRODUCT_OWNER__YANDEX				'Y'
#define MANUFACTURER__FENDA					"FD"

#define SV1_FIELD_SIZE__PRODUCT_MODEL		4
#define SV1_FIELD_SIZE__MANUFACTURER		2
#define SV1_FIELD_SIZE__PRODUCT_OWNER		1

#define SV2_OFFSET__CHECKSUM				124

#define SV2__FIELDS_COUNT					10
#define SV2__EEPROM_SIZE					128

enum sv2_fields {
	STRUCT_V,
	PRODUCT_OWNER,
	MANUFACTURER,
	PRODUCT_MODEL,
	BOARD_CODE,
	REVISION_HI,
	REVISION_MID,
	REVISION_LO,
	REVISION_ADDITIONAL,
	ASSEMBLY,
	DATE,
	SERIAL,
	SV2_FIELDS_COUNT
};

enum board_code_t {
	BOARD_CODE__SOM = 1,
	BOARD_CODE__BACKPLANE,
	BOARD_CODE__LED_DISPLAY,
	BOARD_CODE__LED_RING,
	BOARD_CODE__MICS,
	BOARD_CODE__DC_ETHERNET,
	BOARD_CODE__UNKNOWN,
};

enum parse_res_t {
	PARSE_RES__SUCCESS = 0,
	PARSE_RES__EMPTY,
	PARSE_RES__WRONG_CRC,
	PARSE_RES__UNKNOWN_VERSION,
	PARSE_RES__OBSOLETE_PVT,
	PARSE_RES__OTHER,
};

struct pcba_id
{
	unsigned struct_version;
	char product_owner[MAX_FIELD_SIZE];
	unsigned product_model;
	char manufacturer[MAX_FIELD_SIZE];
	unsigned board_code;
	struct board_revision {
		unsigned hi;
		unsigned mid;
		unsigned lo;
		char additional[MAX_FIELD_SIZE]; // for pcba_id v2
	} board_revision;
	char assembly[MAX_FIELD_SIZE]; // for pcba_id v2
	char date[FIELD_SIZE__DATE + 1];
	unsigned incremental_number;
	unsigned checksum;
};

enum parse_res_t parse_eeprom(unsigned i2c_address, struct pcba_id *pcba_id);

#endif /* _PCBA_ID_H_ */
