/* SPDX-License-Identifier: GPL-2.0-only */
/*
 * Copyright (C) 2020 Yandex Inc
 */

#ifndef _GOWIN__H
#define _GOWIN__H


#define SIZE__RESPONSE_STRUCT		32

#define SIZE__CMD			1
#define SIZE__CRC			2
#define SIZE__SHOW_PIC_DELAY		2
#define SIZE__BRIGHTESS			1
#define SIZE__ANIMATION_TYPE		1
#define SIZE__SET_ANIMATION_RES		15
#define SIZE__STATUS_RES		SIZE__RESPONSE_STRUCT
#define SIZE__SHOW_PIC_RES		1
#define SIZE__RESET_TOUCH_STATE_RES	1
#define SIZE__TOUCH_DEBUG_VAL		2
#define SIZE__RESERVED			15

#define SIZE__OVERHEAD			(SIZE__CMD + SIZE__CRC)

#define SIZE__GET_STATUS		(SIZE__OVERHEAD + SIZE__STATUS_RES)
#define SIZE__SHOW_PIC_WITHOUT_BUF	(SIZE__OVERHEAD + SIZE__SHOW_PIC_DELAY + SIZE__SHOW_PIC_RES)
#define SIZE__SAVE_SETTINGS		(SIZE__OVERHEAD)
#define SIZE__SET_BRIGHTNESS		(SIZE__OVERHEAD + SIZE__BRIGHTESS)
#define SIZE__SET_ANIMATION		(SIZE__OVERHEAD + SIZE__ANIMATION_TYPE + SIZE__SET_ANIMATION_RES)
#define SIZE__RESET_TOUCH_STATE		(SIZE__OVERHEAD + SIZE__RESET_TOUCH_STATE_RES)

#define SIZE__RESPONSE			(SIZE__OVERHEAD + SIZE__RESPONSE_STRUCT)
#define SIZE__BOARD_REV			2
#define SIZE__FRAME_OVERFLOWS		4
#define SIZE__CRC_ERRORS		4

#define GLD_APPLICATION_MODE		0x1

#define GLD_SPI_BUS_NUMBER		0
#define GLD_SPI_CS			0
#define GLD_SPI_DEVICE_SPEED		4000000

#define _STR(x) #x
#define STR(x) _STR(x)
#define SPI_DEVICE_NAME(BUS, CS)	("generic_" STR(BUS) ":" STR(CS))
#define GLD_SPI_DEVICE_NAME	SPI_DEVICE_NAME(LED_DEVICE_SPI_BUS_NUMBER, LED_DEVICE_SPI_CS)

#define RETRIES__DEFAULT		5

#define RETRIES__GET_STATUS		RETRIES__DEFAULT
#define RETRIES__SAVE_SETTINGS		RETRIES__DEFAULT
#define RETRIES__SET_BRIGHTNESS		RETRIES__DEFAULT
#define RETRIES__SET_ANIMATION		RETRIES__DEFAULT
#define RETRIES__RESET_TOUCH_STATE	RETRIES__DEFAULT
#define RETRIES__SHOW_PIC		1

enum gld_cmd_status {
    COMMAND_ERROR = 0x1,
    CRC_ERROR = 0x2,
    CMD_OVF_ERROR = 0x4,
    FRM_OVF_ERROR = 0x8,
    SPKT_ERROR = 0x10,
    LPKT_ERROR = 0x20,
};

enum gld_command {
    CMD__GET_STATUS = 0x00,
    CMD__SHOW_PIC = 0x07,
    CMD__SET_BRIGHTNESS = 0x1B,
    CMD__SET_ANIMATION = 0x24,
    CMD__RESET_TOUCH_STATE = 0xC1,
};

struct gld_response {
    u8 mode;
    u8 fw_version;
    u16 board_rev;
    u32 frame_overflows;
    u32 crc_errors;
    u8 frames_in_queue;
    u8 last_cmd_status;

    u16 uf_crc; // User Flash data CRC16
    u16 crc;
};

#endif /* _GOWIN__H */
